import pybrake.metrics as metrics

from pybrake.notifier import Notifier
from .test_helper import get_exception, TestBacklog, MockResponse

route_payload = {
    "routes": [
        {
            "count": 1,
            "sum": 0.006198883056640625,
            "sumsq": 3.842615114990622e-05,
            "tdigest": "AAAAAkAkAAAAAAAAAAAAATvLIAAB",
            "method": "GET",
            "route": "/test",
            "statusCode": 200,
            "time": "2022-09-29T10:10:00Z"
        }
    ]
}

headers = {
    "Content-Type": "application/json",
    "Authorization": "Bearer xxxxxxxxxxxxxxxxxxxxxxx",
}
route_url = "http://localhost:5000/api/v5/projects/0/routes-stats"
error_url = "http://localhost:5000/api/v3/projects/0/notices"

notifier = Notifier()

err = get_exception()
notice = notifier.build_notice(err)


metrics.Error_Backlog = TestBacklog()
metrics.APM_Backlog = TestBacklog()


def test_send_500_status(mocker, caplog):
    resp = MockResponse(resp_data="Error: Internal Server Error", code=500)
    mocker.patch(
        "urllib.request.urlopen",
        return_value=resp
    )

    metrics.send(url=route_url, payload=route_payload, headers=headers)
    assert "airbrake: unexpected response status_code=500" in caplog.text


def test_send_exception(mocker, caplog):
    resp = MockResponse(resp_data="IOError", code=200)
    mocker.patch(
        "urllib.request.urlopen",
        return_value=resp
    )

    metrics.send(url=route_url, payload=route_payload, headers=headers)
    assert "IOError: for tes" in caplog.text


def test_send_success(mocker, caplog):
    resp = MockResponse(resp_data="Done", code=200)
    mocker.patch(
        "urllib.request.urlopen",
        return_value=resp
    )

    assert metrics.send(url=route_url, payload=route_payload, headers=headers) is None


def test_send_json_loads(mocker, caplog):
    resp = MockResponse(resp_data=b'Gone', code=410)
    mocker.patch(
        "urllib.request.urlopen",
        return_value=resp
    )

    metrics.send(url=route_url, payload=route_payload, headers=headers)
    assert "Expecting value" in caplog.text


def test_send_return_message(mocker, caplog):
    resp = MockResponse(resp_data='{"message": "Message in data!"}'.encode("utf-8"), code=410)
    mocker.patch(
        "urllib.request.urlopen",
        return_value=resp
    )

    metrics.send(url=route_url, payload=route_payload, headers=headers)
    assert "Message in data!" in caplog.text


def test_send_without_encode(mocker, caplog):
    resp = MockResponse(resp_data=b"\x81", code=410)
    mocker.patch(
        "urllib.request.urlopen",
        return_value=resp
    )

    metrics.send(url=route_url, payload=route_payload, headers=headers)
    assert "'utf-8' codec can't decode byte 0x81 in position 0" in caplog.text


def test_send_too_many_request(mocker, caplog):
    resp = MockResponse(resp_data="IOError", code=429)
    mocker.patch(
        "urllib.request.urlopen",
        return_value=resp
    )

    assert metrics.send(url=route_url, payload=route_payload,
                        headers=headers) is None


def test_send_notice_unexpected_res(mocker, caplog):
    resp = MockResponse(resp_data='{"ID": "this id the ID"}'.encode(
        "utf-8"), code=200)
    mocker.patch(
        "urllib.request.urlopen",
        return_value=resp
    )

    output = metrics.send_notice(notifier=notifier, url=error_url,
                                 notice=notice, headers=headers)
    assert output["error"] == 'unexpected Airbrake response'


def test_send_notice_too_many_request(mocker, caplog):
    resp = MockResponse(resp_data="Too Many Request", code=429)
    mocker.patch(
        "urllib.request.urlopen",
        return_value=resp
    )

    output = metrics.send_notice(notifier=notifier, url=error_url,
                                 notice=notice, headers=headers)
    assert output["error"] == 'X-RateLimit-Delay header is missing'


def test_send_notice_backlog(mocker, caplog):
    resp = MockResponse(resp_data=b'Gone', code=410)
    mocker.patch(
        "urllib.request.urlopen",
        return_value=resp
    )

    output = metrics.send_notice(notifier=notifier, url=error_url,
                                 notice=notice, headers=headers)
    assert "Expecting value" in caplog.text
    assert str(output["error"]) == "Expecting value: line 1 column 1 (char 0)"


def test_send_notice_unexpected(mocker, caplog):
    resp = MockResponse(resp_data=b'Gone', code=310)
    mocker.patch(
        "urllib.request.urlopen",
        return_value=resp
    )

    output = metrics.send_notice(notifier=notifier, url=error_url,
                                 notice=notice, headers=headers)
    assert str(output["error"]) == "airbrake: unexpected response status_code=310"


def test_send_notice_return_id(mocker, caplog):
    resp = MockResponse(resp_data='{"id": "this id the ID"}'.encode(
        "utf-8"), code=200)
    mocker.patch(
        "urllib.request.urlopen",
        return_value=resp
    )

    output = metrics.send_notice(notifier=notifier, url=error_url,
                                 notice=notice, headers=headers)
    assert "this id the ID" in output["id"]


def test_send_notice_without_encode(mocker, caplog):
    resp = MockResponse(resp_data=b"\x81", code=410)
    mocker.patch(
        "urllib.request.urlopen",
        return_value=resp
    )

    output = metrics.send_notice(notifier=notifier, url=error_url,
                                 notice=notice, headers=headers)
    assert "'utf-8' codec can't decode byte 0x81 in position 0" in str(output[
        "error"])


def test_send_notice_ioerror(mocker, caplog):
    resp = MockResponse(resp_data="IOError", code=200)
    mocker.patch(
        "urllib.request.urlopen",
        return_value=resp
    )

    output = metrics.send_notice(notifier=notifier, url=error_url,
                                 notice=notice, headers=headers)
    assert "IOError: for tes" in str(output[
        "error"])

metrics.APM_Backlog = None
metrics.Error_Backlog = None



import time
import traceback
import typing as t

from falcon import App

from ..metrics import (
    set_active as set_active_metric,
    get_active as get_active_metric,
    start_span, end_span)
from ..notifier import Notifier
from ..route_metric import RouteMetric

try:
    from sqlalchemy import event
except ImportError:
    _sqla_available = False
else:
    _sqla_available = True

try:
    from jinja2 import Template
except ImportError:
    _jinja_available = False
else:
    _jinja_available = True

try:
    import falcon_auth  # pylint: disable=unused-import
except ImportError:
    _falcon_auth_available = False
else:
    _falcon_auth_available = True

try:
    import falcon_auth2  # pylint: disable=unused-import
except ImportError:
    _falcon_auth2_available = False
else:
    _falcon_auth2_available = True

_UNKNOWN_ROUTE = "UNKNOWN"


def before_request_middleware(notifier, req):
    if notifier and not notifier.config.get("performance_stats"):
        return

    metric = RouteMetric(method=req.method)
    set_active_metric(metric)


def after_request_middleware(notifier, req, resp):
    if notifier and not notifier.config.get("performance_stats"):
        return
    metric = get_active_metric()

    if req.uri_template:
        route = req.uri_template
    elif req.url:
        route = req.url
    else:
        route = _UNKNOWN_ROUTE
    if metric is not None:
        metric.route = route
        metric.status_code = int(resp.status.split(" ")[0]) if isinstance(
            resp.status, str) else resp.status
        metric.content_type = resp.content_type
        metric.end_time = time.time()
        notifier.routes.notify(metric)
        set_active_metric(None)


class PybrakeMiddleware:

    def __init__(self, notifier):
        self.notifier = notifier

    def process_request(self, req, resp):
        before_request_middleware(self.notifier, req=req)
        return resp

    async def process_request_async(self, req, resp):
        before_request_middleware(self.notifier, req=req)
        return resp

    def process_response(self, req, resp, resource, req_succeeded):
        after_request_middleware(notifier=self.notifier, req=req, resp=resp)
        return resp

    async def process_response_async(self, req, resp, resource, req_succeeded):
        after_request_middleware(notifier=self.notifier, req=req, resp=resp)
        return resp


def _before_sql_cursor(notifier):
    def _sqla_before_cursor_execute(
            conn, cursor, statement, parameters, context, executemany
    ):
        if notifier and not notifier.config.get("performance_stats"):
            return
        start_span("sql")

    return _sqla_before_cursor_execute


def _after_sql_cursor(notifier):
    def _sqla_after_cursor_execute(
            conn, cursor, statement, parameters, context, executemany
    ):
        if notifier and not notifier.config.get("performance_stats"):
            return
        end_span("sql")
        metric = get_active_metric()
        if metric is not None:
            try:
                traceback_frm = traceback.extract_stack(limit=7)[0]
            except IndexError as er:  # pylint: disable=unused-variable
                traceback_frm = None
            notifier.queries.notify(
                query=statement,
                method=getattr(metric, "method", ""),
                route=getattr(metric, "route", ""),
                start_time=metric.start_time,
                end_time=time.time(),
                function=traceback_frm.name if traceback_frm else '',
                file=traceback_frm.filename if traceback_frm else '',
                line=traceback_frm.lineno if traceback_frm else 0,
            )

    return _sqla_after_cursor_execute


def request_filter(request, notice):
    if request is None:
        return notice

    ctx = notice["context"]
    ctx["method"] = request.method
    ctx["url"] = request.uri_template
    ctx["route"] = request.url

    try:
        user_addr = request.access_route[0]
    except IndexError:
        user_addr = request.remote_addr
    if user_addr:
        ctx["userAddr"] = user_addr

    if _falcon_auth_available:
        curr_user = request.context['user']
        user = {}
        for s in ["username", "name"]:
            if hasattr(curr_user, s):
                user[s] = getattr(curr_user, s)
        ctx["user"] = user

    if _falcon_auth2_available:
        curr_user = request.context.auth["user"]
        user = {}
        for s in ["username", "name"]:
            if hasattr(curr_user, s):
                user[s] = getattr(curr_user, s)
        ctx["user"] = user

    notice["params"]["request"] = dict(
        json=request.params,
        cookies=dict(request.cookies),
        headers=dict(request.headers),
        environ=request.env,
        url_rule=request.uri_template,
    )

    return notice


def init_app(app: App, config: dict, sqlEngine=None) -> (App, dict):
    if "pybrake" in config:
        raise ValueError("pybrake is already injected")
    if "PYBRAKE" not in config:
        raise ValueError("config['PYBRAKE'] is not defined")

    notifier = Notifier(**config['PYBRAKE'])

    config["pybrake"] = notifier
    old_handle_exception = App._handle_exception

    def _patch_handle_exception(self, req, resp, ex, params):
        notice = notifier.build_notice(ex)
        notice = request_filter(req, notice)
        notifier.send_notice(notice)
        return old_handle_exception

    App._handle_exception = _patch_handle_exception

    # Patch of route stats
    app.add_middleware(PybrakeMiddleware(notifier=notifier))

    # Patch of route breakdown
    if _jinja_available:
        old_template_render = Template.render

        def patch_template_render(self, *args: t.Any, **kwargs: t.Any):
            start_span("template")
            res = old_template_render(self, *args, *kwargs)
            end_span("template")
            return res

        Template.render = patch_template_render

    if sqlEngine and _sqla_available:
        event.listen(sqlEngine, "before_cursor_execute",
                     _before_sql_cursor(notifier))
        event.listen(sqlEngine, "after_cursor_execute",
                     _after_sql_cursor(notifier))

    return app, config



from setuptools import setup, find_packages


def readme():
    with open('README.md') as f:
        return f.read()


# Get version
exec(open('src/pybrake/constant.py').read())

setup(
    name='pybrake',
    version=version,
    description='Python exception notifier for Airbrake',
    long_description=readme(),
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Software Development',
    ],
    keywords='airbrake exception error notifier',
    project_urls={
        'Documentation': 'https://airbrake.io',
        'Funding': 'https://airbrake.io',
        'Say Thanks!': 'https://airbrake.io',
        'Source': 'http://github.com/airbrake/pybrake',
        'Tracker': 'http://github.com/airbrake/pybrake/issues',
    },
    url='http://github.com/airbrake/pybrake',
    author='Airbrake Technologies, Inc.',
    author_email='support@airbrake.io',
    license='MIT',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    install_requires=['tdigest'],
    tests_require=['tdigest'],
    include_package_data=True,
    zip_safe=False
)



import requests
from datetime import datetime
from pycnic.core import Handler

from sqlalchemy import create_engine, Column, Integer, String, Sequence, \
    select, insert
from sqlalchemy.ext.declarative import declarative_base

from pybrake.middleware.pycnic import PybrakeEnableWSGI

engine = create_engine('sqlite:///:memory:', echo=True)
Base = declarative_base()

city_list = ["austin", "pune", "santabarbara", "washington"]


class Entity(Base):
    __tablename__ = 'entity'
    id = Column(Integer, Sequence('id_seq'), primary_key=True)
    name = Column(String(50))

    def __init__(self, name):
        self.name = name

    def __repr__(self):
        return "<Entity('%d', '%s')>" % (self.id, self.name)


class Hello(Handler):

    def get(self, name="World"):
        with engine.begin() as conn:
            stmt = select(Entity)
            result = conn.execute(stmt)
            res = result.scalar()
            if not res:
                stmt = insert(Entity).values(name=name)
                conn.execute(stmt)
        return {"message": "Hello, Welcome to the Weather App!"}


class Date(Handler):

    def get(self):
        return {
            "date": "Current date and time is: %s" % datetime.now()
        }


class Locations(Handler):

    def get(self):
        return {
            "locations": city_list
        }


class Weather(Handler):

    def get(self, location_name):
        if location_name not in city_list:
            return {
                'error': 'Not found: Location not found!'
            }
        with requests.get(
                'https://airbrake.github.io/weatherapi/weather/' +
                location_name) as f:
            return f.json()


class App(PybrakeEnableWSGI):
    config = {
        'PYBRAKE': dict(
            project_id=99999,
            project_key='xxxxxxxxxxxxxxxxxxxxxxxxxxx',
        )
    }
    sqlDBEngine = engine
    
    debug = True
    routes = [
        ("/", Hello()),
        ("/date", Date()),
        ("/locations", Locations()),
        ("/weather/([\w]+)", Weather()),
    ]


if __name__ == "__main__":

    from wsgiref.simple_server import make_server
    Base.metadata.create_all(engine)
    try:
        print("Serving on 0.0.0.0:3000...")
        make_server('127.0.0.1', 3000, App).serve_forever()
    except KeyboardInterrupt:
        pass



import asyncio
import inspect
import time
import traceback
import typing as t

from aiohttp import web

from ..metrics import (
    set_active as set_active_metric,
    get_active as get_active_metric,
    start_span, end_span)
from ..notifier import Notifier
from ..route_metric import RouteMetric

try:
    from jinja2 import Template
except ImportError:
    pass
else:
    # Patch of route breakdown
    old_template_render = Template.render


    def patch_template_render(self, *args: t.Any, **kwargs: t.Any):
        start_span("template")
        res = old_template_render(self, *args, *kwargs)
        end_span("template")
        return res


    Template.render = patch_template_render

try:
    from sqlalchemy import event
except ImportError:
    _sqla_available = False
else:
    _sqla_available = True


def _before_sql_cursor(notifier):
    def _sqla_before_cursor_execute(
            conn, cursor, statement, parameters, context,
            executemany
    ):
        if not notifier.config.get("performance_stats"):
            return
        start_span("sql")

    return _sqla_before_cursor_execute


def _after_sql_cursor(notifier):
    def _sqla_after_cursor_execute(
            conn, cursor, statement, parameters, context,
            executemany
    ):
        if not notifier.config.get("performance_stats"):
            return
        end_span("sql")
        metric = get_active_metric()
        if metric is not None:
            try:
                traceback_frm = traceback.extract_stack(limit=7)[0]
            except IndexError as er:  # pylint: disable=unused-variable
                traceback_frm = None
            notifier.queries.notify(
                query=statement,
                method=getattr(metric, "method", ""),
                route=getattr(metric, "route", ""),
                start_time=metric.start_time,
                end_time=time.time(),
                function=traceback_frm.name if traceback_frm else '',
                file=traceback_frm.filename if traceback_frm else '',
                line=traceback_frm.lineno if traceback_frm else 0,
            )

    return _sqla_after_cursor_execute


def pybrake_middleware(overrides=None, sqlEngine=None):
    if overrides is None:
        overrides = {}

    async def factory(app, handler):
        init_pybrake(app, sqlEngine)

        async def middleware(request: web.Request):
            resp = None
            notifier = app["pybrake"]
            if notifier.config.get("performance_stats"):
                metric = RouteMetric(method=request.method, route=request.path)
                set_active_metric(metric)
            try:
                resp = await handler(request)
                override = overrides.get(resp.status)
                if override:
                    resp = await override(request)
                return resp

            except web.HTTPException as ex:
                override = overrides.get(ex.status)
                if override:
                    resp = await override(request)
                raise ex

            except asyncio.CancelledError as cancelErr:
                raise cancelErr

            except Exception as err:
                handle_exception(err, app["pybrake"], request)
                override = overrides.get(500)
                if override:
                    resp = await override(request)
                raise err
            finally:
                if notifier.config.get("performance_stats"):
                    metric = get_active_metric()
                    if metric is not None:
                        metric.status_code = resp.status if resp else 500
                        metric.content_type = \
                            resp.content_type if resp else request.content_type
                        metric.end_time = time.time()
                        notifier.routes.notify(metric)
                        set_active_metric(None)

        return middleware

    return factory


def init_pybrake(app, sqlEngine=None):
    if "PYBRAKE" not in app:
        raise ValueError("app['PYBRAKE'] is not defined")
    if "pybrake" not in app:
        app["pybrake"] = Notifier(**app["PYBRAKE"])
        if sqlEngine and _sqla_available:
            event.listen(sqlEngine, "before_cursor_execute",
                         _before_sql_cursor(app['pybrake']))
            event.listen(sqlEngine, "after_cursor_execute",
                         _after_sql_cursor(app['pybrake']))


def handle_exception(ex, notifier, request):
    notice = notifier.build_notice(ex)
    notice["context"].update(additional_context(request))
    notice["params"].update(get_headers(request))
    notifier.send_notice(notice)


def additional_context(request):
    return dict(
        userAgent=get_user_agent(request),
        userAddr=attr_from_request(request, "remote"),
        httpMethod=attr_from_request(request, "method"),
        url=attr_from_request(request, "url"),
    )


def get_user_agent(request):
    headers = get_headers(request)
    return headers.get("User-Agent") if headers else None


def get_headers(request):
    headers = attr_from_request(request, "headers")
    return dict(headers=headers) if headers else {}


def attr_from_request(request, attr_name):
    if hasattr(request, attr_name):
        attr = getattr(request, attr_name)
        return attr() if inspect.ismethod(attr) else attr
    return None



from datetime import datetime
from wsgiref.simple_server import make_server

import requests
from pybrake.middleware.turbogears import init_app
from sqlalchemy import Column, Integer, String
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import scoped_session, sessionmaker
from tg import MinimalApplicationConfigurator
from tg import expose, TGController
from tg.configurator.components.sqlalchemy import \
    SQLAlchemyConfigurationComponent
from tg.exceptions import HTTPNotFound
from tg.util import Bunch

DeclarativeBase = declarative_base()

city_list = ["pune", "austin", "santabarbara", "washington"]


class User(DeclarativeBase):
    __tablename__ = 'user'
    id = Column(Integer, primary_key=True)
    username = Column(String(80), unique=True, nullable=False)
    email = Column(String(120), unique=True, nullable=False)

    def __repr__(self):
        return "<User %r>" % self.username


# RootController of our web app, in charge of serving content for /
class RootController(TGController):

    @expose("index.xhtml")
    def index(self):
        count = DBSession.query(User).count()
        if not count:
            user = User()
            user.username = "test"
            user.email = "test@test.com"
            DBSession.add(user)
        return dict(app="Weather")

    @expose("json")
    def date(self):
        return dict(date="Current Date and Time is: %s" % datetime.now())

    @expose("json")
    def locations(self):
        return dict(locations=city_list)

    @expose("json")
    def weather(self, location_name):
        if location_name not in city_list:
            raise HTTPNotFound(detail="Location not found!")
        with requests.get(
                'https://airbrake.github.io/weatherapi/weather/' + location_name) as f:
            return f.json()


# Configure a new minimal application with our root controller.
config = MinimalApplicationConfigurator()
config.register(SQLAlchemyConfigurationComponent)
DBSession = scoped_session(sessionmaker(autoflush=True, autocommit=False))


def init_model(engine):
    DBSession.configure(bind=engine)
    DeclarativeBase.metadata.create_all(engine)


config.update_blueprint({
    'root_controller': RootController(),
    'renderers': ['json', 'kajiki'],  # Enable json in expose
    'default_renderer': 'kajiki',
    'use_sqlalchemy': True,
    'sqlalchemy.url': 'sqlite:///:memory:',
    'model': Bunch(
        DBSession=DBSession,
        init_model=init_model
    ),
    'PYBRAKE': {
        "project_id": 99999,
        "project_key": "xxxxxxxxxxxxxxxxxxxxxxxxxxxx",
    },
})


if __name__ == "__main__":
    # Serve the newly configured web application.
    print("Serving on port 8000...")
    config = init_app(config)
    httpd = make_server('', 8000, config.make_wsgi_app())
    httpd.serve_forever()



from masonite.providers import Provider
from ..tasks.WeatherTest import WeatherTest
from pybrake.middleware.masonite import (
    PybrakeNotifier, PybrakeErrorListener, PybrakeRouteMiddleware
)


class AppProvider(Provider):
    def __init__(self, application):
        self.application = application

    def register(self):
        self.application.bind(
            "pybrake", PybrakeNotifier(self.application)
        )
        self.application.make("event").listen("masonite.exception.*",
                                              [PybrakeErrorListener])
        self.application.make("middleware").add([PybrakeRouteMiddleware])
        self.application.make('scheduler').add(
            WeatherTest().every_minute()
        )

    def boot(self):
        pass



from contextvars import ContextVar
from datetime import *

import requests
from pybrake.middleware.sanic import init_app
from sanic import Sanic
from sanic import response
from sqlalchemy import INTEGER, Column, String, select, insert
from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy.ext.asyncio import create_async_engine
from sqlalchemy.orm import declarative_base
from sqlalchemy.orm import sessionmaker

app = Sanic(__name__)

bind = create_async_engine("sqlite+aiosqlite:///./test.db", echo=True)
Base = declarative_base()

app.config["PYBRAKE"] = dict(
    project_id=9999,
    project_key="xxxxxxxxxxxxxxxxxxxxxxxxx",
    environment="test",
    performance_stats=True,  # False to disable APM
)

_base_model_session_ctx = ContextVar("session")

app = init_app(app, bind)


@app.middleware("request")
async def inject_session(request):
    request.ctx.session = sessionmaker(bind, AsyncSession,
                                       expire_on_commit=False)()
    request.ctx.session_ctx_token = _base_model_session_ctx.set(
        request.ctx.session)


@app.middleware("response")
async def close_session(request, response):
    if hasattr(request.ctx, "session_ctx_token"):
        _base_model_session_ctx.reset(request.ctx.session_ctx_token)
        await request.ctx.session.close()


def setup_database():
    @app.listener('after_server_start')
    async def connect_to_db(*args, **kwargs):
        async with bind.begin() as conn:
            await conn.run_sync(Base.metadata.create_all)

    @app.listener('after_server_stop')
    async def disconnect_from_db(*args, **kwargs):
        async with bind.begin() as conn:
            await conn.close()


class User(Base):
    __tablename__ = "user"

    id = Column(INTEGER, primary_key=True)
    username = Column(String(80), unique=True, nullable=False)
    email = Column(String(120), unique=True, nullable=False)

    def __repr__(self):
        return "<User %r>" % self.username


city_list = ["pune", "austin", "santabarbara", "washington"]


# API for Hello Application
@app.route("/")
async def run(request):
    session = request.ctx.session
    async with session.begin():
        stmt = select(User)
        result = await session.execute(stmt)
        res = result.scalar()
        if not res:
            stmt = insert(User).values(username="test", email="test@test.com")
            await session.execute(stmt)
    return response.html("""
        <html lang="en">
            <head>
              <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
              <meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1.0"/>
              <title>Cherrypy Weather App</title>
            </head>
            <body>
                <H1>Hello, Welcome to Cherrypy Weather App.</H1>
            </body>
        </html>
    """)


# API for current server date
@app.route("date")
def getdate(request):
    return response.json({
        'date': "Current Date and Time is: %s" % datetime.now()
    })


# API for location details
@app.route("locations")
def get_location_details(request):
    return response.json({'locations': city_list})


# API for weather details for a location
@app.route("/weather/<location_name>")
def get_weather_details(request, location_name):
    if location_name not in city_list:
        raise response.json({'error': 'Location not found!'}, 400)
    with requests.get(
            'https://airbrake.github.io/weatherapi/weather/' + location_name) as f:
        return response.json(f.json())


# debug logs enabled with debug = True
if __name__ == "__main__":
    setup_database()
    app.run(host="0.0.0.0", port=3000, debug=True, auto_reload=True)



import os
from datetime import datetime
import requests

from starlette.applications import Starlette
from starlette.responses import JSONResponse, HTMLResponse
from starlette.routing import Route
from pybrake.middleware.starlette import init_app

from sqlalchemy import Column, Integer, String
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

DIRECTORY = os.path.dirname(os.path.realpath(__file__))
city_list = ["pune", "austin", "santabarbara", "washington"]

# SQL setup
SQLALCHEMY_DATABASE_URL = "sqlite:///:memory:"

engine = create_engine(
    SQLALCHEMY_DATABASE_URL, connect_args={"check_same_thread": False}
)
SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)

Base = declarative_base()

db = SessionLocal()


class User(Base):
    __tablename__ = 'user'
    id = Column(Integer, primary_key=True)
    username = Column(String(80), unique=True, nullable=False)
    email = Column(String(120), unique=True, nullable=False)

    def __repr__(self):
        return "<User %r>" % self.username


Base.metadata.create_all(bind=engine)


async def index(request):
    db.query(User).all()
    html_content = """
            <html>
                <head>
                    <title>Welcome to the Starlette weather app.</title>
                </head>
                <body>
                    <h1>Hello, Welcome to the Starlette weather app.</h1>
                </body>
            </html>
            """
    return HTMLResponse(content=html_content, status_code=200)


async def date(req):
    return JSONResponse({"date": "Current Date and Time is: %s" % datetime.now()})


async def locations(req):
    return JSONResponse({"locations": city_list} )


async def weather(req):
    location_name = req.path_params['location_name']
    if location_name not in city_list:
        return JSONResponse(content={
            "message": "Location not found!"
        }, status_code=404)
    with requests.get(
            'https://airbrake.github.io/weatherapi/weather/' + location_name
    ) as f:
        return JSONResponse(f.json())


app = Starlette(debug=True, routes=[
    Route('/', index),
    Route('/date', date),
    Route('/locations', locations),
    Route('/weather/{location_name}', weather),
])


app.state.PYBRAKE = {
    "project_id": 99999,
    "project_key": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
}

app = init_app(app, engine)



# -*- coding: utf-8 -*-
"""Template Helpers used in weather-fullstack."""
import logging
from markupsafe import Markup
from datetime import datetime

log = logging.getLogger(__name__)


def current_year():
    now = datetime.now()
    return now.strftime('%Y')


def icon(icon_name):
    return Markup('<i class="glyphicon glyphicon-%s"></i>' % icon_name)


# Import commonly used helpers from WebHelpers2 and TG
from tg.util.html import script_json_encode

try:
    from webhelpers2 import date, html, number, misc, text
except SyntaxError:
    log.error("WebHelpers2 helpers not available with this Python Version")



from pybrake.remote_settings import RemoteSettings
from pybrake.constant import version as __version__


def test_build_remote_setting_poll_url():
    config = {
        "error_notifications": True,
        "performance_stats": True,
        "query_stats": True,
        "queue_stats": True,
        "error_host": "https://api.airbrake.io",
        "apm_host": "https://api.airbrake.io",
    }
    res = RemoteSettings(project_id=403427,
                         host="https://notifier-configs.airbrake.io",
                         config=config)
    assert 'https://notifier-configs.airbrake.io/2020-06-18/config/403427' \
           '/config.json?notifier_name=pybrake&notifier_version=' + \
           __version__ in res._poll_url(res._data)


def test_build_remote_setting_error_notifications():
    """
    To see what happens if error_notifications is set to false, run this test.
    :return:
    """
    config = {
        "error_notifications": False,
        "performance_stats": True,
        "query_stats": True,
        "queue_stats": True,
        "error_host": "https://api.airbrake.io",
        "apm_host": "https://api.airbrake.io",
    }
    res = RemoteSettings(project_id=403427,
                         host="https://notifier-configs.airbrake.io",
                         config=config)
    assert res._process_error_notifications(res._data) is None


def test_build_remote_setting_performance_stats():
    """
    To see what happens if performance_stats is set to false, run this test.
    :return:
    """
    config = {
        "error_notifications": True,
        "performance_stats": False,
        "query_stats": True,
        "queue_stats": True,
        "error_host": "https://api.airbrake.io",
        "apm_host": "https://api.airbrake.io",
    }
    res = RemoteSettings(project_id=403427,
                         host="https://notifier-configs.airbrake.io",
                         config=config)
    assert res._process_performance_stats(res._data) is None
